Package: mortar
Type: Package
Title: Standardize Data Science Workflows
Version: 0.4.0
Authors@R: 
    c(
    person("Joseph","Zemmels",email="jzemmels@usgs.gov",role = c("aut"),comment = c(ORCID = "0009-0008-1463-6313")),
    person("Anthony","Martinez",email="ajmartinez@usgs.gov",role = c("aut","cre"),comment = c(ORCID = "0000-0002-4295-0261")),
    person("Jesse","Ross",role = c("ctb"),comment = c(ORCID = "0000-0002-5422-8284"))
    )
Maintainer: Anthony Martinez <ajmartinez@usgs.gov>
Description: Helper functions to standardizes common workflows in the USGS Data
    Science Community of Practice to produce more robust, reproducible
    pipelines. It contains helper functions to standardize (1) the organization
    of project repositories and (2) the creation ofpipelines from the 'targets'
    R Package (Landau et al. (2026) <doi:10.5281/zenodo.18555866>), using the DS
    CoP best practices. We draw upon community developed best practices as well
    as certain USGS-specific requirements. See 
    Shrycock et al. (2023) <doi:10.3133/tm7B2> for examples of these USGS
    requirements.
License: CC0
BugReports: https://github.com/DOI-USGS/mortar/issues
URL: https://doi-usgs.github.io/mortar/,
        https://github.com/DOI-USGS/mortar,
        https://code.usgs.gov/water/computational-tools/mortar
Encoding: UTF-8
Imports: cli, gert, glue, purrr, R.utils, rlang, usethis
RoxygenNote: 7.3.2
Suggests: knitr, renv, rmarkdown, stringr, tarchetypes, targets,
        testthat (>= 3.0.0), withr
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-24 14:17:59 UTC; ajmartinez
Author: Joseph Zemmels [aut] (ORCID: <https://orcid.org/0009-0008-1463-6313>),
  Anthony Martinez [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4295-0261>),
  Jesse Ross [ctb] (ORCID: <https://orcid.org/0000-0002-5422-8284>)
Repository: CRAN
Date/Publication: 2026-04-28 19:00:15 UTC
Built: R 4.5.2; ; 2026-05-01 18:53:52 UTC; unix

# Generated by roxygen2: do not edit by hand

export(file_edit)
export(get_usgs_gitlab_url)
export(tar_init)
export(use_changelog_usgs)
export(use_code_json_usgs)
export(use_code_of_conduct_usgs)
export(use_contributing_usgs)
export(use_disclaimer_approved_usgs)
export(use_disclaimer_provisional_usgs)
export(use_gitignore_usgs)
export(use_gitlab_mr_template_usgs)
export(use_license_usgs)
export(use_project_usgs)
export(use_readme_rmd_usgs)
export(use_readme_usgs)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# mortar <img src="man/figures/logo.svg" width="20%" align="right" alt="Hex logo for mortar package. Shows a partially filled water pipe coming out of a brick wall where there is generous amounts of mortar between bricks." />

<!-- badges: start -->

<!-- badges: end -->

## Overview

> *“If you need to copy + paste the same code more than once, write a
> function. If you need to write the same function more than once,
> create a package.”*
>
> –DRY coding principle

`mortar` standardizes common workflows in the USGS Data Science
Community of Practice (DS CoP) to produce more robust, reproducible
pipelines. It contains helper functions to standardize (1) the
organization of project repositories and (2) the creation of
[`targets`](https://books.ropensci.org/targets/) pipelines using the DS
CoP best practices. We draw upon community developed best practices as
well as certain USGS-specific requirements.

### `use_project_usgs()`

Use `use_project_usgs()` to setup your project repository. It will
create all the metadata files necessary for a software release, with
options to customize the files. These files can then be manually edited
throughout the development process to keep metadata up to date.

### `tar_init()`

If you’re setting up a `targets` pipeline project, you can **also** run
`tar_init()` to set up the `targets` directory structure we commonly use
in the DS CoP. `tar_init()` generates the targets script file
(`_targets.R`), split phase directories (optional; e.g., `01_fetch/`),
and subdirectories (e.g., `02_process/src/`).

Refer to the [Get
started](https://doi-usgs.github.io/mortar/articles/mortar.html) page
for more information using `mortar.`

## Installation

Install mortar from CRAN with

``` r
install.packages("mortar")
```

You can also install the development version from source from
[GitHub](https://github.com/DOI-USGS/mortar):

``` r
# Install the pak package if you don't already have it
#install.packages("pak")

pak::pak("DOI-USGS/mortar")
```

## Contributing

Please consider reporting bugs, asking questions, or suggesting
enhancements. This is a community developed package and we want it to be
inclusive of the needs of our community. If considering contributing,
please read
[CONTRIBUTING.md](https://github.com/DOI-USGS/mortar/blob/main/CONTRIBUTING.md)
for more information. Also, please read through
[CONDUCT.md](https://github.com/DOI-USGS/mortar/blob/main/CONDUCT.md) to
ensure that we maintain a warm, welcoming, safe, and professional
environment.

**NOTE: this is a template created using the `mortar` package. The contents below are for illustrative purposes and should be edited to reflect your use case. This message can be deleted in acknowledgement that you've added this file to your project.**

# Project title

**Authors**: First M. Last, Another M. Author.

**Point of contact**: First M. Last ([fmlast\@usgs.gov](mailto:fmlast@usgs.gov))

**Repository Type**: *R* and *Python* scripts supporting publication

**Year of Origin**: 2026

**Year of Version**: 2026

**Digital Object Identifier (DOI)**: <https://doi.org/10.5066/xxxxxxxx>

**USGS Information Product Data System (IPDS) no.**: IP-xxxxxx (internal agency tracking)

------------------------------------------------------------------------

[Provide a  short, one or two sentence description of your repository.]

## Purpose

[Short description of why this repository exists.]

## Repository organization

-   `.gitlab/` directory for files used by GitLab
        - `merge_request_templates` directory containing templates for merge request descriptions.
            - `Default.md` a file containing the suggested default merge request template.
-   `.gitignore` file specifying which files should not be tracked in git repository.
-   `CHANGELOG.md` description of changes to each versions of the software.
-   `CODE_OF_CONDUCT.md` information on the contribution code of conduct.
-   `code.json` metadata file containing information for code contained in this repository. Compliant with [USGS template](https://www.usgs.gov/products/software/software-management/distribution-usgs-code).
-   `CONTRIBUTING.md` information on how to contribute to this repository.
-   `DISCLAIMER.md` provisional USGS disclaimer of code in repository. Should be replaced with approved version when approved.
-   `LICENSE.md` license information for code.
-   `README.md` general information about the project.

## Setup

[A step by step series of examples that tell you have to get a development environment running.]

## Running the code

[Instructions describing how to run the code. May be used instead of or in addition to **Deployment** section below.]

## Building and testing

[Instructions go here.]

## Deployment

[Additional notes about how to deploy this on a server, if applicable. **Do not include any credentials, IP addresses, or other sensitive information.**]

## Code runtime

[Give a rough estimate for how long the code is expected to run, including a brief description of the setup the the runtime is estimated from.]

## Expertise required

[What sort of expertise must a user have to use code in this repository?]

## Built With

[Provide a bulleted list of languages, frameworks, tools used, with link and use description. Below are examples.]

-   [R](https://www.r-project.org/about.html) - statistical modeling
-   [Python](https://www.python.org/) - data parsing
-   [Jupyter Notebook](https://jupyter.org/) - documentation and sharing

## Contributing

Please read [CONTRIBUTING.md](CONTRIBUTING.md) for details on the process for submitting pull requests to us. Please read [CODE_OF_CONDUCT.md](CODE_OF_CONDUCT.md) for details on adhering by the [USGS Code of Scientific Conduct](https://www.usgs.gov/office-of-science-quality-and-integrity/fundamental-science-practices).

## Authors

-   [**First Last**](https://www.usgs.gov/staff-profiles/first-last) - *Job title or role* - [Some Water Science Center](https://www.usgs.gov/centers/some-water-science-center)
-   [**Another Author**](https://www.usgs.gov/staff-profiles/another-author) - *Job title or role* - [USGS Water Mission Area](https://www.usgs.gov/mission-areas/water-resources)

## License

This project is licensed under the Creative Commons CC0 1.0 Universal License - see the [LICENSE.md](LICENSE.md) file for details.

## Suggested Citation

In the spirit of open source, please cite any re-use of the source code stored in this repository. Below is the suggested citation:

> Last FM, Author AM. 2026. Project title. U.S. Geological Survey software release. Reston, Va. <https://doi.org/10.5066/xxxxxxx>.

## Associated products

[List citations for associated works, such as associated publications or data releases, input data sets, or previous versions of software release.]

## Acknowledgments

- A tip of the hat to anyone whose code was used or who helped but is not an author.

#' Set up a USGS project directory
#'
#' @description Wraps the other \code{use_*_usgs} functions to initialize
#'   multiple files in one directory.
#'
#' @param home chr, root directory of targets project. To use the current
#'   working directory, use  \code{"."}.
#' @param gitignore_additions chr vector, other files/directories to be added to
#'   .gitignore or `NULL` (default) for no additions.
#' @param readme_type chr, either "md" to create README as a markdown file
#'   (README.md; default) or "rmd" to create it as an R markdown file
#'   (README.Rmd). Using an Rmd file will allow you to include R code and output
#'   in your README.md.
#' @param disclaimer_type chr, either "provisional" (default) to include the
#'   provisional software disclaimer statement or "approved" for the approved
#'   disclaimer statement. It is important to only include the approved
#'   disclaimer statement if your software is an approved software release.
#' @param repo_url chr, URL to Git repository. If `NULL` (default) a generic
#'   URL will be provided as a link to the repository in CONTRIBUTING.md.
#'   Otherwise, the issue page for `repo_url` will be used as the link.
#' @param use_mr_template lgl, Should GitLab Merge Request template be included
#'   in project repository. Default is `TRUE`.
#' @param open lgl; whether to open the files for interactive editing.
#'
#' @returns \code{NULL} invisibly
#'
#' @examples
#' tmp <- tempdir()
#' unlink(tmp, recursive = TRUE, force = TRUE)
#' dir.create(tmp)
#'
#' use_project_usgs(
#'   home = tmp,
#'   repo_url = "https://code.usgs.gov/water/computational-tools/mortar"
#' )
#' list.files(tmp)
#'
#' # start over
#' unlink(tmp, recursive = TRUE, force = TRUE)
#' dir.create(tmp)
#'
#' # creates README.Rmd and DISCLAIMER_APPROVED instead
#' use_project_usgs(
#'   home = tmp,
#'   readme_type = "rmd",
#'   disclaimer_type = "approved",
#'   repo_url = "https://code.usgs.gov/water/computational-tools/mortar"
#' )
#' list.files(tmp)
#'
#' # Clean up temp files
#' unlink(tmp, recursive = TRUE, force = TRUE)
#'
#' @export
use_project_usgs <- function(
  home,
  gitignore_additions = NULL,
  readme_type = c("md", "rmd"),
  disclaimer_type = c("provisional", "approved"),
  repo_url = get_usgs_gitlab_url("origin"),
  use_mr_template = TRUE,
  open = rlang::is_interactive()
) {
  disclaimer_type <- rlang::arg_match(disclaimer_type)
  readme_type <- rlang::arg_match(readme_type)
  if (!any(rlang::is_scalar_character(repo_url) | is.null(repo_url))) {
    cli::cli_abort(c(
      "{.arg repo_url} must be a character with length 1 or NULL.",
      x = "You provided a class {.cls {class(repo_url)}} of length {length(repo_url)}."
    ))
  }
  if (!rlang::is_scalar_logical(use_mr_template)) {
    cli::cli_abort(c(
      "{.arg use_mr_template} must be a logical with length 1.",
      x = "You provided a class {.cls {class(use_mr_template)}} of length {length(use_mr_template)}."
    ))
  }
  if (!rlang::is_scalar_logical(open)) {
    cli::cli_abort(c(
      "{.arg open} must be a logical with length 1.",
      x = "You provided a class {.cls {class(open)}} of length {length(open)}."
    ))
  }
  if (!(is.character(gitignore_additions) | is.null(gitignore_additions))) {
    cli::cli_abort(c(
      "x" = "{.arg gitignore_additions} must be a character vector or {.code NULL}, not class {.cls {class(gitignore_additions)}}."
    ))
  }

  # README file
  if (readme_type == "rmd") {
    use_readme_rmd_usgs(home = home, open = open)
  } else {
    use_readme_usgs(home = home, open = open)
  }

  # DISCLAIMER file
  if (disclaimer_type == "approved") {
    use_disclaimer_approved_usgs(home = home, open = open)
  } else {
    use_disclaimer_provisional_usgs(home = home, open = open)
  }

  if (use_mr_template) {
    use_gitlab_mr_template_usgs(home = home, open = open)
  }

  # code.json
  use_code_json_usgs(home = home, open = open)

  # CHANGELOG
  use_changelog_usgs(home = home, open = open)

  # CONTRIBUTING
  use_contributing_usgs(home = home, repo_url = repo_url, open = open)

  # CODE OF CONDUCT
  use_code_of_conduct_usgs(home = home, open = open)

  # .gitignore
  use_gitignore_usgs(
    home = home,
    additions = gitignore_additions,
    open = open
  )
}

#' Add individual USGS project files to a directory
#'
#' @description Creates common project files like .gitignore, README.md. LICENSE.md,
#'   etc.#'
#'
#' @param home chr, root directory of targets project. To use the current
#'   working directory, use  \code{"."}.
#' @param additions chr vector, other files/directories to be added to
#'   .gitignore
#' @param open lgl; whether to open the file for interactive editing
#'
#' @returns \code{NULL} invisibly
#'
#' @examples
#' tmp <- tempdir()
#'
#' use_gitignore_usgs(home = tmp,
#'                    additions = c("excluded_file.R",
#'                                  "excluded_dir",
#'                                  "*excluded_pattern*"))
#' # here are the contents of the .gitignore:
#' cat(readLines(file.path(tmp,".gitignore")), sep = "\n")
#'
#' unlink(tmp, recursive = TRUE, force = TRUE)
#'
#' @name use-file-usgs
#' @rdname use-file-usgs
#' @export
use_gitignore_usgs <- function(
  home,
  additions = NULL,
  open = rlang::is_interactive()
) {
  use_file_usgs(
    home = home,
    inst_file = "GITIGNORE",
    out_file = ".gitignore",
    additions = additions,
    open = open
  )

  return(invisible(NULL))
}

#' @rdname use-file-usgs
#' @export
use_readme_usgs <- function(home, open = rlang::is_interactive()) {
  use_file_usgs(
    home = home,
    inst_file = "README",
    out_file = "README.md",
    additions = NULL,
    open = open
  )

  return(invisible(NULL))
}

#' @rdname use-file-usgs
#' @export
use_readme_rmd_usgs <- function(home, open = rlang::is_interactive()) {
  use_file_usgs(
    home = home,
    inst_file = "README",
    out_file = "README.Rmd",
    additions = NULL,
    open = open
  )

  # add YAML heading as in usethis::use_readme_rmd
  cat(
    c(
      "---",
      "output: github_document",
      "---",
      " ",
      "<!-- README.md is generated from README.Rmd. Please edit that file -->",
      " ",
      readLines(file.path(home, "README.Rmd"))
    ),
    file = file.path(home, "README.Rmd"),
    sep = "\n",
    fill = FALSE
  )

  # add git hook that forces README.md to be updated if README.Rmd is updated.
  repo <- tryCatch(gert::git_find(usethis::proj_get()), error = function(e) {
    NULL
  })
  if (!is.null(repo)) {
    # copied + pasted from usethis:::render_template("readme-rmd-pre-commit.sh")
    readme_rmd_pre_commit_sh <-
      c(
        "#!/bin/bash",
        "README=($(git diff --cached --name-only | grep -Ei '^README\\.[R]?md$'))",
        "MSG=\"use 'git commit --no-verify' to override this check\"",
        "",
        "if [[ ${#README[@]} == 0 ]]; then",
        "  exit 0",
        "fi",
        "",
        "if [[ README.Rmd -nt README.md ]]; then",
        "  echo -e \"README.md is out of date; please re-knit README.Rmd\\n$MSG\"",
        "  exit 1",
        "elif [[ ${#README[@]} -lt 2 ]]; then",
        "  echo -e \"README.Rmd and README.md should be both staged\\n$MSG\"",
        "  exit 1",
        "fi"
      )

    usethis::use_git_hook(
      hook = "pre-commit",
      script = readme_rmd_pre_commit_sh
    )
  }

  return(invisible(NULL))
}

#' @rdname use-file-usgs
#' @export
use_license_usgs <- function(home, open = rlang::is_interactive()) {
  use_file_usgs(
    home = home,
    inst_file = "LICENSE",
    out_file = "LICENSE.md",
    additions = NULL,
    open = open
  )

  return(invisible(NULL))
}

#' @rdname use-file-usgs
#' @export
use_code_json_usgs <- function(home, open = rlang::is_interactive()) {
  use_file_usgs(
    home = home,
    inst_file = "CODE_JSON",
    out_file = "code.json",
    additions = NULL,
    open = open
  )

  return(invisible(NULL))
}

#' @rdname use-file-usgs
#' @export
use_disclaimer_provisional_usgs <- function(
  home,
  open = rlang::is_interactive()
) {
  use_file_usgs(
    home = home,
    inst_file = "DISCLAIMER_PROVISIONAL",
    out_file = "DISCLAIMER_PROVISIONAL.md",
    additions = NULL,
    open = open
  )

  return(invisible(NULL))
}

#' @rdname use-file-usgs
#' @export
use_disclaimer_approved_usgs <- function(
  home,
  open = rlang::is_interactive()
) {
  use_file_usgs(
    home = home,
    inst_file = "DISCLAIMER_APPROVED",
    out_file = "DISCLAIMER_APPROVED.md",
    additions = NULL,
    open = open
  )

  return(invisible(NULL))
}

#' @rdname use-file-usgs
#' @export
use_code_of_conduct_usgs <- function(
  home,
  open = rlang::is_interactive()
) {
  use_file_usgs(
    home = home,
    inst_file = "CODE_OF_CONDUCT",
    out_file = "CODE_OF_CONDUCT.md",
    additions = NULL,
    open = open
  )

  return(invisible(NULL))
}

#' @param repo_url chr, URL to Git repository. If `NULL` (default) a generic
#'   URL will be provided as a link to the repository in CONTRIBUTING.md.
#'   Otherwise, the issue page for `repo_url` will be used as the link.
#' @rdname use-file-usgs
#' @export
use_contributing_usgs <- function(
  home,
  repo_url = NULL,
  open = rlang::is_interactive()
) {
  use_file_usgs(
    home = home,
    inst_file = "CONTRIBUTING",
    out_file = "CONTRIBUTING.md",
    additions = NULL,
    open = FALSE
  )

  # Add repo issue link if applicable
  if (!is.null(repo_url)) {
    file_edit(
      file = file.path(home, "CONTRIBUTING.md"),
      txt = glue::glue("[1]: {repo_url}/-/issues"),
      match = ~ grepl("\\[1]: https://gitlab.com/namespace/repo/-/issues", .x),
      append = FALSE
    )
  }

  # Open file ----
  usethis::edit_file(path = file.path(home, "CONTRIBUTING.md"), open = open)

  return(invisible(NULL))
}

#' @rdname use-file-usgs
#' @export
use_changelog_usgs <- function(home, open = rlang::is_interactive()) {
  use_file_usgs(
    home = home,
    inst_file = "CHANGELOG",
    out_file = "CHANGELOG.md",
    additions = NULL,
    open = open
  )

  return(invisible(NULL))
}

#' @rdname use-file-usgs
#' @export
use_gitlab_mr_template_usgs <- function(home, open = FALSE) {
  dir_path <- file.path(".gitlab", "merge_request_templates")

  if (!dir.exists(file.path(home, dir_path))) {
    dir.create(file.path(home, dir_path), recursive = TRUE)
  }

  use_file(
    home = home,
    inst_file = "MR_TEMPLATE",
    inst_subdir = "template_files",
    out_file = file.path(dir_path, "Default.md"),
    additions = NULL,
    open = open
  )

  return(invisible(NULL))
}

#' Get the HTTPS URL for the git remote of a repo at code.usgs.gov
#'
#' @param remote_name chr; name of the git remote. The default is "origin".
#'
#' @returns a character sting of the HTTPS URL to the remote repo
#' @export
#'
#' @examples
#' \dontrun{
#' # Will fail if working directory is not in a git repo
#' get_usgs_gitlab_url("origin")
#' }
get_usgs_gitlab_url <- function(remote_name = "origin") {
  remote_url <- usethis::git_remotes()

  # Ensure remote name is valid
  if (!remote_name %in% names(remote_url)) {
    cli::cli_abort("{.arg {remote_name}} is not an existing remote.")
  }

  remote_url <- remote_url[[remote_name]]

  # Ensure remote is from code.usgs.gov
  if (!grepl(x = remote_url, pattern = "code.usgs.gov")) {
    cli::cli_abort(c(
      "x" = "The git remote was expecting a remote from {.url https://code.usgs.gov}.",
      "i" = "The remote URL is {.url {remote_url}}"
    ))
  }

  # Convert SSH URL to HTTPS URL
  if (grepl(x = remote_url, pattern = "^git@")) {
    remote_url <- gsub(x = remote_url, pattern = ".git$", replacement = "")
    remote_url <- gsub(
      x = remote_url,
      pattern = "^git@code.usgs.gov:",
      replacement = "https://code.usgs.gov/"
    )
  }

  return(remote_url)
}


#' Internal: core function used in other use_*_usgs functions, which all have
#' the same basic structure
#'
#' @param home chr, root directory of targets project. To use the current
#'   working directory, use  \code{"."}.
#' @param inst_file chr; template file to be included
#' @param out_file chr; path to location of output file
#' @param additions chr; additional lines to add to the template file
#' @param open lgl; whether to open the file for interactive editing
#'
#' @returns NULL, invisibly
#' @noRd
#'
use_file_usgs <- function(
  home,
  inst_file,
  out_file = stringr::str_remove(inst_file, "\\.txt"),
  additions = NULL,
  open = rlang::is_interactive()
) {
  # Write file ----
  use_file(
    home = home,
    inst_file,
    inst_subdir = "template_files",
    out_file = out_file,
    additions = additions,
    open = open
  )

  # Inform user of source ----
  cli::cli_inform(c(
    "i" = "Using {.file {out_file}} from mortar template."
  ))

  return(invisible(NULL))
}

testthat::test_that("use_project_usgs() validates its arguments", {
  # Set temporary working directory and removal instructions
  tmp <- withr::local_tempdir()
  old <- setwd(tmp)
  on.exit(setwd(old), add = TRUE)

  # Invalid home directory
  testthat::expect_error(
    use_project_usgs(home = "this/dir/doesn't/exist", repo_url = NULL),
    "`home` must be a path to a directory that exists."
  )
  # Invalid gitignore additions
  testthat::expect_error(
    use_project_usgs(".", gitignore_additions = TRUE, repo_url = NULL),
    "`gitignore_additions` must be a character vector or `NULL`, not"
  )
  # Invalid readme_type should result in an error mentioning the argument
  testthat::expect_error(
    use_project_usgs(".", readme_type = "txt", repo_url = NULL),
    '`readme_type` must be one of "md" or "rmd", not "txt"'
  )
  # Invalid disclaimer_type should result in an error mentioning the argument
  testthat::expect_error(
    use_project_usgs(".", disclaimer_type = "final", repo_url = NULL),
    '`disclaimer_type` must be one of "provisional" or "approved", not "final"'
  )
  # Invalid repo_url
  testthat::expect_error(
    use_project_usgs(".", repo_url = 4L),
    "`repo_url` must be a character with length 1"
  )
  # Invalid use_mr_template
  testthat::expect_error(
    use_project_usgs(".", use_mr_template = 1234, repo_url = NULL),
    "`use_mr_template` must be a logical with length 1"
  )
  # Invalid open
  testthat::expect_error(
    use_project_usgs(".", open = "TRUE", repo_url = NULL),
    "`open` must be a logical with length 1"
  )
  # Valid options should not fail
  testthat::expect_invisible(
    suppressMessages(use_project_usgs(".", repo_url = NULL, open = FALSE))
  )
})

testthat::test_that("use_project_usgs() creates expected files when templates exist", {
  # Skip this test entirely if the template directory isn't found.  Without
  # these files, the helper functions that write files will error because
  # system.file() can't locate the templates.
  template_dir <- system.file("template_files", package = "mortar")
  testthat::skip_if_not(dir.exists(template_dir))

  # Ensure at least one required template is present; otherwise there is
  # nothing to copy into the project directory.  README is the most
  # fundamental template so we check for it.
  testthat::skip_if_not(file.exists(file.path(template_dir, "README")))

  # Set temporary working directory and removal instructions
  tmp <- withr::local_tempdir()
  old <- setwd(tmp)
  on.exit(setwd(old), add = TRUE)

  # Run the function.  We specify open = FALSE to avoid launching an
  # editor during the test.  The `repo_url` argument is provided so that
  # CONTRIBUTING.md contains a link to the issues page.  Passing
  # `gitignore_additions` allows us to verify that our custom entries are
  # written to .gitignore.
  testthat::expect_no_error(
    suppressMessages(use_project_usgs(
      home = tmp,
      gitignore_additions = c("myfile.txt"),
      readme_type = "md",
      disclaimer_type = "provisional",
      repo_url = "https://code.usgs.gov/test/repo",
      use_mr_template = TRUE,
      open = FALSE
    ))
  )

  # After execution, the expected files should exist in the project directory.
  expected_files <- c(
    "README.md",
    "DISCLAIMER_PROVISIONAL.md",
    "code.json",
    "CHANGELOG.md",
    "CONTRIBUTING.md",
    "CODE_OF_CONDUCT.md",
    ".gitignore",
    ".gitlab/merge_request_templates/Default.md"
  )
  # Make sure each of these files exists
  for (f in expected_files) {
    testthat::expect_true(file.exists(file.path(tmp, f)))
  }
  # The .gitignore file should contain the custom addition we supplied.
  gitignore_lines <- readLines(file.path(tmp, ".gitignore"))
  testthat::expect_true(any(grepl("myfile.txt", gitignore_lines)))
})

testthat::test_that("use_project_usgs() with R Markdown README and approved disclaimer", {
  # Skip if the necessary templates aren't available.
  template_dir <- system.file("template_files", package = "mortar")
  testthat::skip_if_not(dir.exists(template_dir))
  testthat::skip_if_not(file.exists(file.path(template_dir, "README")))

  # Set temporary working directory and removal instructions
  tmp <- withr::local_tempdir()

  # Create project files
  suppressMessages(testthat::expect_no_error(
    use_project_usgs(
      home = tmp,
      gitignore_additions = NULL,
      readme_type = "rmd",
      disclaimer_type = "approved",
      repo_url = "https://code.usgs.gov/test/repo",
      use_mr_template = FALSE,
      open = FALSE
    )
  ))

  # Check that README.Rmd and the approved disclaimer exist.
  testthat::expect_true(file.exists(file.path(tmp, "README.Rmd")))
  testthat::expect_true(file.exists(file.path(tmp, "DISCLAIMER_APPROVED.md")))
  testthat::expect_false(
    file.exists(file.path(tmp, ".gitlab/merge_request_templates/Default.md"))
  )

  # The README.Rmd file should start with a YAML header that sets
  # `output: github_document`.  We only look at the first few lines to
  # confirm this; additional content comes from the template.
  rmd_lines <- readLines(file.path(tmp, "README.Rmd"))
  testthat::expect_true(length(rmd_lines) >= 3)

  # Check that YAML header starts with ---
  testthat::expect_identical(rmd_lines[1], "---")
  # The second line should specify output format.  We use grepl() because
  # whitespace may vary. YAML header specifies github_document output
  testthat::expect_true(grepl("output: github_document", rmd_lines[2]))
})

testthat::test_that("use_contributing_usgs() inserts and omits issue links correctly", {
  # Skip if contributing template doesn't exist.
  template_dir <- system.file("template_files", package = "mortar")
  testthat::skip_if_not(dir.exists(template_dir))
  testthat::skip_if_not(file.exists(file.path(template_dir, "CONTRIBUTING")))

  # Test with repo_url supplied: link to issues should be added
  tmp1 <- withr::local_tempdir()
  old <- setwd(tmp1)
  on.exit(setwd(old), add = TRUE)

  # Use open = FALSE to prevent opening editor
  suppressMessages(testthat::expect_no_error(
    use_contributing_usgs(
      home = tmp1,
      repo_url = "https://code.usgs.gov/wma/example",
      open = FALSE
    )
  ))
  contrib_path1 <- file.path(tmp1, "CONTRIBUTING.md")
  testthat::expect_true(file.exists(contrib_path1)) # repo_url in CONTRIBUTING.md
  lines1 <- readLines(contrib_path1)
  # The inserted link should appear somewhere in the file; we won't rely on
  # its exact position because line numbers may change if the template
  # evolves.
  testthat::expect_true(
    any(grepl("https://code.usgs.gov/wma/example/-/issues", lines1))
  )

  # Test without repo_url: no link should be inserted - retain generic link
  tmp2 <- withr::local_tempdir()
  suppressMessages(testthat::expect_no_error(
    use_contributing_usgs(home = tmp2, repo_url = NULL, open = FALSE)
  ))
  contrib_path2 <- file.path(tmp2, "CONTRIBUTING.md")
  testthat::expect_true(file.exists(contrib_path2))
  lines2 <- readLines(contrib_path2)
  testthat::expect_true(
    any(grepl("https://gitlab.com/namespace/repo/-/issues", lines2))
  )
})

testthat::test_that("use_file_usgs() errors when the template file is not found", {
  # use_file_usgs() should fail if inst_file is not in the template directory.
  # This test uses a temporary directory as home; since the error arises
  # before any file is created, there's no need to clean up.
  tmp <- withr::local_tempdir()
  old <- setwd(tmp)
  on.exit(setwd(old), add = TRUE)

  testthat::expect_error(
    use_file_usgs(
      inst_file = "NON_EXISTENT_TEMPLATE",
      out_file = "dummy.txt",
      home = tmp,
      open = FALSE
    ),
    "`inst_file` must be one of .*"
  )
})

testthat::test_that("get_usgs_gitlab_url() fails gracefully for unknown remotes", {
  # Skip this test if usethis is not available.  get_usgs_gitlab_url() uses
  # usethis::git_remotes() to retrieve remote names.  If usethis isn't
  # installed or if not using a git repo, we can't exercise this behavior.
  testthat::skip_if_not_installed("usethis")
  testthat::skip_if_not(tryCatch(gert::git_find(), error = \(e) FALSE))
  # Provide a remote name that is extremely unlikely to exist.  We don't
  # assert on the error message itself because it may change; it's
  # sufficient to know that the function throws an error.
  testthat::expect_error(
    get_usgs_gitlab_url("INVALID_Ample_Breakfast_Earthenware"),
    "`INVALID_Ample_Breakfast_Earthenware` is not an existing remote."
  )
})

#' Internal: core function used to use a file from the inst directory
#'
#' @param inst_file chr; path within inst directory to the template file to be
#'   included
#' @param out_file chr; path to location of output file
#' @param home chr, root directory of project. Defaults to current working
#'   directory
#' @param additions chr; additional lines to add to the template file
#' @param open lgl; whether to open the file for interactive editing
#'
#' @returns \code{NULL} invisibly
#' @noRd
#'
use_file <- function(
  inst_file,
  inst_subdir = "",
  out_file = stringr::str_remove(inst_file, "\\.txt"),
  home = ".",
  additions = NULL,
  open = rlang::is_interactive()
) {
  # Check arguments ----
  if (!dir.exists(system.file(inst_subdir, package = "mortar"))) {
    cli::cli_abort(c(
      "x" = "{.arg inst_subdir} must be a subdirectory of inst that exists."
    ))
  }

  rlang::arg_match(
    inst_file,
    list.files(system.file(inst_subdir, package = "mortar"))
  )

  if (!all(rlang::is_scalar_character(home), dir.exists(home))) {
    cli::cli_abort(c(
      "x" = "{.arg home} must be a path to a directory that exists."
    ))
  }

  if (!rlang::is_scalar_character(out_file)) {
    cli::cli_abort(c(
      "x" = "{.arg out_file} must be a character vector, not class {.cls {class(out_file)}}."
    ))
  }

  if (!(is.character(additions) | is.null(additions))) {
    cli::cli_abort(c(
      "x" = "{.arg additions} must be a character vector or {.code NULL}, not class {.cls {class(additions)}}."
    ))
  }

  # Write file ----
  out_file_path <- file.path(home, out_file)

  if (!file.exists(out_file_path)) {
    file.create(out_file_path)
  }

  inst_file_path <- file.path(inst_subdir, inst_file)

  cat(
    c(
      readLines(system.file(inst_file_path, package = "mortar")),
      additions
    ),
    file = out_file_path,
    sep = "\n",
    fill = FALSE
  )

  # Open file ----
  usethis::edit_file(path = out_file_path, open = open)

  return(invisible(NULL))
}

testthat::test_that(
  "use_file writes to the correct file path", {
    # Set temporary working directory and removal instructions
    tmp <- withr::local_tempdir()
    old <- setwd(tmp)
    on.exit(setwd(old), add = TRUE)
    out_file <- "out_test.txt"

    # Test
    suppressMessages(
      use_file(
        inst_file = "GITIGNORE",
        inst_subdir = "template_files",
        out_file = out_file,
        additions = NULL,
        open = FALSE
      )
    )

    # Check that the file was created in the correct location
    testthat::expect_true(file.exists(file.path(tmp, out_file)))
  }
)

testthat::test_that(
  "use_file throws an error for invalid files/dirs", {
    # Set temporary working directory and removal instructions
    tmp <- withr::local_tempdir()
    old <- setwd(tmp)
    on.exit(setwd(old), add = TRUE)

    # Error if `home` is invalid
    testthat::expect_error(
      use_file(
        inst_file = "GITIGNORE",
        inst_subdir = "template_files",
        out_file = "out_test.txt",
        home = "invalid_dir",
        additions = NULL
      ),
      "`home` must be a path to a directory that exists."
    )

    # Error if `inst_subdir` is invalid
    testthat::expect_error(
      use_file(
        inst_file = "GITIGNORE",
        inst_subdir = "invalid_dir",
        out_file = "out_test.txt",
        additions = NULL
      ),
      "`inst_subdir` must be a subdirectory of inst that exists."
    )

    # Error if inst_file invalid
    testthat::expect_error(
      use_file(
        inst_file = "invalid_file",
        inst_subdir = "template_files",
        out_file = "out_test.txt",
        additions = NULL
      ),
      '`inst_file` must be one of .* not "invalid_file".'
    )
  }
)

# Tests for use_file() and USGS wrappers with explicit negative-path coverage.
testthat::test_that(
  "use_file writes a template plus additions; open=TRUE triggers mocked editor", {
    # Set temporary working directory and removal instructions
    tmp <- withr::local_tempdir()
    old <- setwd(tmp)
    on.exit(setwd(old), add = TRUE)

    readme_out <- file.path(tmp, "README.md")

    suppressMessages(use_file(
      inst_file = "README",
      inst_subdir = "template_files",
      out_file = "README.md",
      home = tmp,
      additions = c("ADD1","ADD2"),
      open = FALSE
    ))

    testthat::expect_true(file.exists(readme_out))
    testthat::expect_equal(
      tail(readLines(readme_out), 2),
      c("ADD1","ADD2")
    )
  }
)

#' Initialize a targets project
#'
#' @description Run this function in a new targets project directory. It will
#'   create R files and directories using the targets project structure often
#'   used by the USGS Data Science Community of Practice. For more information,
#'   internal USGS employees can look at section 12 of Targets 2 training
#'   course.
#'
#' @param home chr, root directory of targets project. To use the current
#'   working directory, use  \code{"."}.
#' @param phase_names chr vector, names of target phases like "fetch",
#'   "process", etc.
#' @param phase_nums int vector, numbers to prepend to phase_names like
#'   \code{"1_fetch"}, \code{"2_process"}, etc. Defaults to
#'   seq_along(phase_names)
#' @param separate_phase_scripts lgl, should a different R script be created for
#'   each phase like "1_fetch.R", "2_process.R", etc? If FALSE, then targets
#'   lists will be initialized in _targets.R file. Defaults to TRUE
#' @param phase_subdirs chr vector, subdirectories within each phase. Defaults
#'   to "src", "out"
#' @param use_leading_zeros lgl, should "0" be used in front of phase numbers in
#'   file and directory names?
#' @param overwrite lgl, should the initialization overwrite files and folders
#'   that already exist? Defaults to FALSE
#'
#' @examples
#' # temporary directories in which targets project is initialized (you can skip
#' # this part if creating your own project)
#' tmp <- tempdir()
#' unlink(tmp, recursive = TRUE, force = TRUE)
#' dir.create(tmp)
#'
#' # creates 1_fetch, 2_process, 3_summarize R scripts and directories
#' tar_init(home = tmp, phase_names = c("fetch", "process", "summarize"))
#'
#' list.files(tmp, full.names = FALSE, recursive = TRUE, all.files = TRUE,
#'            pattern = "\\.(R|empty)$")
#'
#' # clean out tmp folder
#' unlink(tmp, recursive = TRUE, force = TRUE)
#' dir.create(tmp)
#'
#' # different structure starting with 0_config and including "in/" dir in each phase
#' tar_init(home = tmp,
#'          phase_names = c("config", "pull", "munge", "visualize"),
#'          phase_nums = 0:3,
#'          phase_subdirs = c("in", "src", "out"),
#'          separate_phase_scripts = FALSE)
#'
#' list.files(tmp, full.names = FALSE, recursive = TRUE, all.files = TRUE,
#'            pattern = "\\.(R|empty)$")
#'
#' # clean out tmp folder
#' unlink(tmp, recursive = TRUE, force = TRUE)
#' dir.create(tmp)
#'
#' @returns \code{NULL} invisibly
#' @export
tar_init <- function(
  home,
  phase_names,
  phase_nums = seq_along(phase_names),
  separate_phase_scripts = TRUE,
  phase_subdirs = c("src", "out"),
  use_leading_zeros = FALSE,
  overwrite = FALSE
) {
  # Check arguments ----
  if (!is.character(phase_names)) {
    cli::cli_abort(c(
      "x" = "{.arg phase_names} must be a character vector, not class {.cls {class(phase_names)}}."
    ))
  }

  if (!rlang::is_integerish(as.numeric(phase_nums))) {
    cli::cli_abort(c(
      "x" = "{.arg phase_nums} must be an integer-like vector, not class {.cls {class(phase_nums)}}."
    ))
  }

  if (!all(rlang::is_scalar_character(home), dir.exists(home))) {
    cli::cli_abort(c(
      "x" = "{.arg home} must be a character path (length 1) to a directory that exists."
    ))
  }

  if (!rlang::is_scalar_logical(separate_phase_scripts)) {
    cli::cli_abort(c(
      "x" = "{.arg separate_phase_scripts} must be logical (length 1), not class {.cls {class(separate_phase_scripts)}} (length {length(separate_phase_scripts)})."
    ))
  }

  if (!is.character(phase_subdirs)) {
    cli::cli_abort(c(
      "x" = "{.arg phase_subdirs} must be a character vector not class {.cls {class(phase_subdirs)}}."
    ))
  }

  if (!rlang::is_scalar_logical(overwrite)) {
    cli::cli_abort(c(
      "x" = "{.arg overwrite} must be logical (length 1), not class {.cls {class(overwrite)}} (length {length(overwrite)})."
    ))
  }

  if (!rlang::is_scalar_logical(use_leading_zeros)) {
    cli::cli_abort(c(
      "x" = "{.arg use_leading_zeros} must be logical (length 1), not class {.cls {class(use_leading_zeros)}} (length {length(use_leading_zeros)})."
    ))
  }

  if (length(phase_names) != length(phase_nums)) {
    cli::cli_abort(c(
      "x" = "{.arg phase_nums} must be the same length as {.arg phase_names}.",
      "!" = "{.arg phase_nums} has a length of {length(phase_nums)} and {.arg phase_names} has a length of {length(phase_names)}."
    ))
  }

  # Create phase R files and directories ----
  # create "#_phase" R files and directories, adding phase_subdirs and .empty
  # files in each

  # Add leading zeros (if applicable)
  phase_nums_targets <- phase_nums
  phase_nums_files <- Vectorize(ifelse)(
    use_leading_zeros,
    sprintf("%02d", phase_nums),
    phase_nums
  )

  # Normalize home directory
  if (home == ".") {
    home <- ""
  } else if (!endsWith(home, "/")) {
    home <- paste0(home, "/")
  }

  ## Create subdirectories ----
  subdir_paths <- expand.grid(
    dir = glue::glue("{home}{phase_nums_files}_{phase_names}"),
    subdir = phase_subdirs
  )
  subdir_paths <- glue::glue_data(.x = subdir_paths, "{dir}/{subdir}")

  purrr::walk(subdir_paths, ~ dir_setup(.x, overwrite = overwrite))

  ## Create phase scripts (if applicable) ----
  phase_files <- glue::glue("{home}{phase_nums_files}_{phase_names}.R")
  phase_dirs <- glue::glue("{home}{phase_nums_files}_{phase_names}")
  if (separate_phase_scripts & (!all(file.exists(phase_files)) | overwrite)) {
    phase_file_text <- glue::glue(
      "p{phase_nums_targets}_targets_list <- list()"
    )
    purrr::walk2(phase_file_text, phase_files, ~ cat(.x, file = .y))
  }

  # Create targets file ----
  if (!file.exists("_targets.R") | overwrite) {
    if (separate_phase_scripts) {
      phase_function_text <- glue::glue(
        "# tar_source(c({glue::glue_collapse(glue::double_quote(phase_dirs), sep = \", \")}))"
      )
      phase_script_text <- glue::glue(
        "# Load target list files\n",
        "tar_source(c({glue::glue_collapse(glue::double_quote(phase_files),",
        "sep = \", \")}))\n\n"
      )
      phase_targets <- glue::glue_collapse(
        glue::glue("p{phase_nums_targets}_targets_list"),
        sep = ", "
      )
      phase_target_text <- glue::glue("c({phase_targets})")
    } else {
      phase_script_text <- ""
      phase_target_text <- "list()"
      phase_function_text <- glue::glue("# tar_source(\"{home}\")")
    }

    cat(
      glue::glue(
        "# Load packages used to define the pipeline itself",
        "library(targets)\n",

        "# Set options here like `packages = c(\"dplyr\")",
        "tar_option_set()\n",

        "# Load function definition scripts",
        "{phase_function_text}\n",

        "{phase_script_text}",

        "# Define pipeline",
        "{phase_target_text}",
        "\n",
        .sep = "\n"
      ),
      file = glue::glue("{home}_targets.R")
    )
  }

  return(invisible(NULL))
}

#' Internal: create a directory (with .empty file) if it doesn't exist
#'
#' @param dir_path chr; path to directory to create
#' @param overwrite lgl; if directory exists, should it be overwritten?
#'
#' @returns lgl; did file creation succeed
#' @noRd
#'
dir_setup <- function(dir_path, overwrite) {
  if (!dir.exists(dir_path) | overwrite) {
    unlink(dir_path, recursive = TRUE)
    dir.create(dir_path, recursive = TRUE)
  }

  file.create(paste0(dir_path, "/.empty"))
}

# Simple tests to make sure that errors work as expected.
testthat::test_that("Errors thrown in `tar_init()`", {
  # Set temporary working directory and removal instructions
  tmp <- withr::local_tempdir()
  old <- setwd(tmp)
  on.exit(setwd(old), add = TRUE)

  testthat::expect_error(
    tar_init(".", c("a", "b", "c"), phase_nums = 1:2),
    "`phase_nums` must be the same length as `phase_names`"
  )
  testthat::expect_error(
    tar_init(".", phase_names = "a", separate_phase_scripts = c(TRUE, FALSE)),
    "`separate_phase_scripts` must be logical"
  )
  testthat::expect_error(
    tar_init(".", phase_names = "a", overwrite = c(TRUE, FALSE)),
    "`overwrite` must be logical"
  )
  testthat::expect_error(
    tar_init(".", phase_names = "a", use_leading_zeros = "TRUE"),
    "`use_leading_zeros` must be logical"
  )
  testthat::expect_error(
    tar_init(".", phase_names = "a", phase_subdirs = TRUE),
    "`phase_subdirs` must be a character vector"
  )
  testthat::expect_no_error(
    tar_init(".", c("a", "b"), phase_nums = 1:2)
  )
})

# Tests for tar_init() with deeper verification of _targets.R content.
# We verify:
#  - argument validation (basic smoke checks)
#  - directory creation for phases/subdirs
#  - exact/ordered content expectations inside _targets.R for both modes:
#       * separate_phase_scripts = TRUE
#       * separate_phase_scripts = FALSE
testthat::test_that("tar_init creates expected structure with separate_phase_scripts = TRUE", {
  # Set temporary working directory and removal instructions
  tmp <- withr::local_tempdir()
  old <- setwd(tmp)
  on.exit(setwd(old), add = TRUE)

  tar_init(
    ".",
    phase_names = c("fetch", "process"),
    phase_nums = c(1, 2),
    separate_phase_scripts = TRUE,
    phase_subdirs = c("src", "out"),
    use_leading_zeros = FALSE,
    overwrite = TRUE
  )

  # Directories exist
  testthat::expect_true(dir.exists("1_fetch/src"))
  testthat::expect_true(dir.exists("1_fetch/out"))
  testthat::expect_true(dir.exists("2_process/src"))
  testthat::expect_true(dir.exists("2_process/out"))

  # Phase scripts exist
  testthat::expect_true(file.exists("1_fetch.R"))
  testthat::expect_true(file.exists("2_process.R"))

  # _targets.R content checks
  testthat::expect_true(file.exists("_targets.R"))
  txt <- readLines("_targets.R")

  # Ensure parts of targets script file are in correct order
  idx_lib <- grep("^library\\(targets)", txt)
  idx_opts <- grep("^# Set options", txt)
  idx_phase <- grep("^# Load target list files", txt)
  idx_def <- grep("^# Define pipeline", txt)

  testthat::expect_true(all(c(idx_lib, idx_opts, idx_phase, idx_def) > 0))
  testthat::expect_lt(idx_lib, idx_opts)
  testthat::expect_lt(idx_opts, idx_phase)
  testthat::expect_lt(idx_phase, idx_def)

  # tar_source line references the individual phase files (quoted, comma-separated)
  line_after_phase <- txt[idx_phase + 1]
  testthat::expect_match(
    line_after_phase,
    'tar_source\\(c\\("1_fetch.R\", \\"2_process.R\\"))'
  )

  # pipeline definition aggregates the phase list objects
  testthat::expect_true(
    any(grepl("^c\\(p1_targets_list, p2_targets_list)$", txt))
  )
})

testthat::test_that("tar_init in non-phase mode (separate_phase_scripts = FALSE) uses list()", {
  tmp <- withr::local_tempdir()
  old <- setwd(tmp)
  on.exit(setwd(old), add = TRUE)

  tar_init(
    ".",
    phase_names = c("config", "run"),
    phase_nums = c(0, 1),
    separate_phase_scripts = FALSE,
    phase_subdirs = c("in", "src", "out"),
    use_leading_zeros = TRUE,
    overwrite = TRUE
  )

  # Ensure proper directories and subdirectories exist
  testthat::expect_true(dir.exists("00_config/in"))
  testthat::expect_true(dir.exists("01_run/out"))
  testthat::expect_true(file.exists("_targets.R"))

  txt <- readLines("_targets.R")
  # When separate_phase_scripts = FALSE, the pipeline definition should be 'list()'
  testthat::expect_true(any(grepl("^list\\()$", txt)))
  # And the 'tar_source' helper should be commented with the project home
  testthat::expect_true(any(
    grepl('^# tar_source\\(\\"\\")$', txt) |
      grepl('^# tar_source\\(".*/?")$', txt)
  ))
})

#' Edit a text file programmatically
#'
#' Insert or transform (`txt` argument) one or more lines in a file specified
#' either by line number or a boolean function (`match` argument). Any file that
#' can be read with the \code{readLines()} function should be compatible.
#'
#' @param file chr, path to a file
#' @param txt chr or chr fun, either a single character string to insert into
#'   the file or a function that transforms a character string. If supplying a
#'   function, the first argument of the function must accept a single string
#'   and it must return a single string. The function will be applied
#'   independently to each line in the file that matches the match argument.
#'   Supports anonymous functions. See examples.
#' @param match num vector or lgl fun, either a numeric vector indicating line
#'   indices in the file or a function that returns a logical indicating lines
#'   to change. If supplying a function, the first argument of the function must
#'   accept a single string and it must return a logical vector. The function
#'   will be applied independently to each line in the file. Supports anonymous
#'   functions. See examples.
#' @param append lgl, if TRUE the text will be added as a new line after the
#'   line(s) specified in the match argument. If FALSE, then the text will
#'   replace the lines specified in the match argument
#'
#' @returns (invisibly) FALSE if no lines in the specified file match the match
#'   argument and TRUE otherwise
#' @export
#'
#' @examples
#' tmpfile <- tempfile()
#'
#' writeLines(text =
#'              c("hello I am joe",
#'                "this is multiple lines of text",
#'                "12345",
#'                "",
#'                "^ line 4 is an empty line"),
#'            con = tmpfile)
#' cat(readLines(tmpfile),sep = "\n")
#'
#' # The simplest use is to append a line in a file with new text. Note that txt
#' # must be a single character string.
#' file_edit(file = tmpfile,
#'           txt = "this line is appended after the second line",
#'           match = 2,
#'           append = TRUE)
#' cat(readLines(tmpfile),sep = "\n")
#'
#' # We can instead replace a line using append = FALSE. Here, we replace the empty
#' # line in the file. After adding the line above, the empty line now occupies
#' # index 5.
#' file_edit(file = tmpfile,
#'           txt = "this line is no longer empty",
#'           match = 5,
#'           append = FALSE)
#' cat(readLines(tmpfile),sep = "\n")
#'
#' # The txt argument can also be a function, anonymous or named. It just needs to
#' # accept a single string and return a single string. Here, we replace "is" with
#' # "was" in line 6.
#' file_edit(file = tmpfile,
#'           txt = ~ stringr::str_replace(.x, "is", "was"),
#'           match = 6,
#'           append = FALSE)
#' cat(readLines(tmpfile),sep = "\n")
#'
#' # The match argument can also be a  function. It just needs to accept a single
#' # string and return a single boolean. You can use it with the txt argument
#' # for some interesting interactions. Here, we change every line containing
#' # the word "is" to uppercase
#' file_edit(file = tmpfile,
#'           txt = toupper,
#'           match = ~ stringr::str_detect(.x, " is "),
#'           append = FALSE)
#' cat(readLines(tmpfile),sep = "\n")
#'
#' # strings containing carriage returns are preserved. Here, we use the readLines
#' # and length functions to ensure we append the string to the end of the file
#' multiline <-
#' "
#' Here is a string
#' that takes up multiple lines"
#'
#' file_edit(file = tmpfile,
#'           txt = multiline,
#'           match = length(readLines(tmpfile)),
#'           append = TRUE)
#' cat(readLines(tmpfile),sep = "\n")
#'
file_edit <- function(file, txt, match, append = TRUE) {
  if (!is.character(file) | length(file) > 1) {
    cli::cli_abort(
      "Argument {.arg file} must be a single string, not a
                   {.cls {class(file)}} of length {length(file)}."
    )
  }
  if (!file.exists(file)) {
    cli::cli_abort("I couldn't find a file at {.file {file}}.")
  }

  # txt must be character or function
  # match must be numeric vector or function

  lines <- readLines(file)

  # either match is a numeric vector
  if (all(is.numeric(match))) {
    if (any(match < 0)) {
      cli::cli_abort(c(
        "{.arg match} cannot contain negative numbers.",
        x = "Negative indices are not valid.",
        i = "If {.arg match} is numeric, it is treated as indices of line numbers."
      ))
    }

    if (any(match == 0) & !append) {
      cli::cli_abort(c(
        "If {.arg match} equals 0, then {.arg append} must be TRUE.",
        i = "When {.arg match} equals 0, it adds text to the beginning of the file.",
        x = "If {.arg append} is FALSE, nothing is appended."
      ))
    }

    if (any(match == 0)) {
      cli::cli_inform(c(
        "{.arg match} == 0 which will result in text being prepended to the beginning of the file."
      ))
    }

    if (!rlang::is_integerish(match)) {
      cli::cli_abort(c(
        "{.arg match} cannot contain non-integer(ish) numbers.",
        x = "Non-integer indices are not valid.",
        i = "If {.arg match} is numeric, it is treated as indices of line numbers."
      ))
    }

    matched_lines <- match
  } else {
    # or its a function that returns a boolean
    match <- rlang::as_function(match)

    if (!rlang::is_function(match)) {
      cli::cli_abort(
        "Invalid argument {.arg match}. This must be a named or
                     anonymous function."
      )
    }

    # determine lines for which match returns TRUE
    matched_lines_list <- purrr::map(lines, match)
    if (
      !all(
        purrr::map_lgl(matched_lines_list, is.logical),
        sum(lengths(matched_lines_list)) == length(matched_lines_list)
      )
    ) {
      cls_out <- as.character(unique(purrr::map(matched_lines_list, class)))
      cli::cli_abort(c(
        "{.arg match} must return a single logical value for each line.",
        i = "If {.arg match} is a function, it is used to perform a logical subset on lines based on string matches.",
        x = "The function provided output(s) in the following class{?es}: {cls_out}.",
        x = "The function provided {sum(lengths(matched_lines_list))} output{?s} for {length(lines)} line{?s}."
      ))
    }
    matched_lines <- as.logical(matched_lines_list)

    # we just need the indices of the lines
    matched_lines <- which(matched_lines)
  }

  if (length(matched_lines) == 0) {
    cli::cli_warn(c(
      "!" = "No lines in {.file {file}} match the {.arg match} argument.",
      "i" = "No changes were made to {.file {file}}."
    ))
    return(invisible(FALSE))
  }

  # either txt is a string or it's a function that returns another string
  if (all(is.character(txt))) {
    if (all(length(txt) > 1, length(txt) != length(matched_lines))) {
      cli::cli_abort(c(
        "The length of {.arg txt} must be equal to 1 or the number of matches identified by {.arg match}.",
        "i" = "{.arg match} resulted in {length(matched_lines)} matched line{?s} and {.arg txt} has a length of {length(txt)}."
      ))
      return(invisible(FALSE))
    }

    new_txt <- txt
  } else {
    txt <- rlang::as_function(txt)

    if (!rlang::is_function(txt)) {
      cli::cli_abort(
        "Invalid argument {.arg txt}. This must be a named or
                     anonymous function."
      )
    }

    # apply the transformation to the matched lines
    new_txt <- purrr::map(lines[matched_lines], txt)

    if (
      any(
        purrr::map_chr(new_txt, class) != "character",
        sum(lengths(new_txt)) != length(matched_lines)
      )
    ) {
      cls_out <- as.character(unique(purrr::map(new_txt, class)))

      cli::cli_abort(c(
        "{.arg txt} must return a single character string for a single input.",
        x = "If {.arg txt} is a function, it is used to transform a character to another character.",
        i = "The function provided output(s) in the following class{?es}: {cls_out}.",
        i = "The function provided {sum(lengths(new_txt))} output{?s} for {length(matched_lines)} matched line{?s}."
      ))
    }

    new_txt <- as.character(new_txt)
  }

  # replace the matched lines if append = FALSE
  if (!append) {
    lines[matched_lines] <- new_txt
  } else {
    # otherwise, insert the new_txt values after each matched line
    lines <- R.utils::insert(
      x = lines,
      ats = matched_lines + 1,
      values = new_txt,
      useNames = FALSE
    )
  }

  writeLines(lines, con = file)
  return(invisible(TRUE))
}

# Why so many cases?
# - file_edit() accepts both numeric indices and matching functions; we exercise
#   both styles.
# - We verify correct behavior for appending vs. replacing, multiple matches,
#   and functional transformations.
# - We also deliberately probe edge cases (e.g., zero/negative indices,
#   mismatched output lengths) to document and protect current behavior.
testthat::test_that("file_edit appends new text after functional match", {
  tmp <- withr::local_tempfile()

  writeLines(c("alpha","beta","gamma", "delta"), tmp)

  testthat::expect_invisible(
    file_edit(
      file = tmp,
      txt  = c("fruit", "veggies"),
      match = ~ grepl("t", .x),
      append = TRUE
    )
  )

  #beta -> fruit inserted after, delta -> veggies inserted after
  testthat::expect_equal(
    readLines(tmp),
    c("alpha", "beta", "fruit", "gamma", "delta", "veggies")
  )

  # Append a literal line after any line containing 'i'
  testthat::expect_invisible(
    file_edit(
      file = tmp,
      txt  = "grains",
      match = ~ grepl("i", .x),
      append = TRUE
    )
  )

  # fruit -> grains inserted after, veggies -> grains inserted after
  testthat::expect_equal(
    readLines(tmp),
    c("alpha", "beta", "fruit", "grains", "gamma", "delta", "veggies", "grains")
  )

  # Fail when there is a mismatch between the number of lines and matches
  testthat::expect_error(
    file_edit(
      file = tmp,
      txt  = c("sweets", "fats"),
      match = ~ grepl("mm", .x),
      append = TRUE
    ),
    "`match` resulted in 1 matched line and `txt` has a length of 2."
  )
})

testthat::test_that(
  "file_edit replaces lines with character vector of equal length", {
    tmp <- withr::local_tempfile()

    writeLines(c("x", "y", "z"), tmp)

    # Replace lines 1 and 3 with different values
    testthat::expect_invisible(
      file_edit(
        file = tmp,
        txt   = c("X","Z"),
        match = c(1,3),
        append = FALSE
      )
    )
    testthat::expect_equal(readLines(tmp), c("X","y","Z"))

    # Fail with txt/match length mismatch
    testthat::expect_error(
      file_edit(
        file = tmp,
        txt   = c("X","Z"),
        match = c(1, 2, 3),
        append = FALSE
      ),
      "`match` resulted in 3 matched lines and `txt` has a length of 2."
    )
  })

testthat::test_that(
  "file_edit with zero/negative indices documents current behaviour", {
    tmp <- withr::local_tempfile()

    writeLines(c("line1", "line2", "line3"), tmp)

    # The implementation treats numeric 'match' as literal indices. If index is
    # 0, it will insert the text before the first line.
    testthat::expect_message(
      file_edit(file = tmp, txt = "ADDED_AFTER_LINE0", match = 0, append = TRUE),
      "`match` == 0 which will result in text being prepended to the beginning of the file."
    )

    testthat::expect_equal(
      readLines(tmp),
      c("ADDED_AFTER_LINE0", "line1", "line2", "line3")
    )

    testthat::expect_error(
      file_edit(file = tmp, txt = "ADDED_AFTER_LINE0", match = 0, append = FALSE),
      "If `match` equals 0, then `append` must be TRUE."
    )

    # Negative index values are not valid.
    testthat::expect_error(
      file_edit(file = tmp, txt = "Negative index", match = -1, append = TRUE),
      "`match` cannot contain negative numbers."
    )
  }
)

testthat::test_that(
  "file_edit errors when match function returns wrong type/length", {
    tmp <- withr::local_tempfile()

    writeLines(c("a","b","c"), tmp)

    # match function must return a single logical per line; returning character
    # should raise an error via cli_abort
    testthat::expect_error(
      file_edit(tmp, txt = "x", match = function(.) "not logical", append = TRUE),
      "The function provided output\\(s) in the following class: character"
    )

    # returning a logical vector of wrong length per line is also invalid
    testthat::expect_error(
      file_edit(
        tmp,
        txt = "x",
        match = function(.) c(TRUE, FALSE),
        append = TRUE
      ),
      "The function provided 6 outputs for 3 lines."
    )
  }
)

testthat::test_that(
  "file_edit errors when txt function returns wrong type/length", {
    tmp <- withr::local_tempfile()

    writeLines(c("one","two","three"), tmp)

    # txt function must return a single string per matched line
    testthat::expect_error(
      file_edit(
        tmp,
        txt   = function(.) c("a","b"),
        match = ~ TRUE,
        append = FALSE
      ),
      "The function provided 6 outputs for 3 matched lines."
    )

    # txt function that returns non-character should error
    testthat::expect_error(
      file_edit(
        tmp,
        txt   = function(.) 123,
        match = ~ TRUE,
        append = FALSE
      ),
      "The function provided output\\(s) in the following class: numeric."
    )
  }
)

testthat::test_that(
  "file_edit handles multiline strings and replace mode", {
    tmp <- withr::local_tempfile()

    writeLines(c("hello","world"), tmp)

    multi <- "multi\nline"
    testthat::expect_invisible(
      file_edit(tmp, txt = multi, match = 1, append = FALSE)
    )

    testthat::expect_equal(readLines(tmp), c("multi", "line", "world"))
  })

---
title: "mortar"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{mortar}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

The goal of `mortar` is to standardize how we USGS Data Science Community of Practice (DS CoP) structure our collaborative projects.
The package's functionality focuses on setting up project directories using the DS CoP conventions.
Think of `mortar` as a collection of helper functions for creating files and folders we often use in our workflows.

```{r setup}
library(mortar)
```

## Set up a USGS git repo with `use_project_usgs()`

You may want or need to include common files in a project's git repo to adhere to coding conventions or standards.
These include a **README** that summarizes basic information about a repo, a **License** that details how the contents of a repo can be used, and others.

The `use_*_usgs()` family of functions create these sorts of files based on USGS-specific templates.
These functions are heavily inspired by similar functions from the [`usethis`](https://usethis.r-lib.org/) R package.

At the top of each of these template files, the following message is included:

> -**NOTE: this is a template created using the `mortar` package. The contents below are for illustrative purposes and should be edited to reflect your use case. This message can be deleted in acknowledgement that you've added this file to your project.**

This acts as a nudge to the author to update the documentation to the specifics of the project. It also acts as a quality control measure, allowing code authors and reviewers to ensure that they have at least opened and looked at each of these files.

For example, you can add a .gitignore file to your repo with some pre-set (often forgotten) ignores using the following.
If you don't like the contents of these template files, you're able to customize it using or `additions` argument or to edit them afterward to your needs.

```{r eval=FALSE}
use_gitignore_usgs(additions = c("scratch*", "*/tmp/*"))
```

This creates the standard .gitignore with additional lines for `scratch*` and `*/tmp/*`.
Run ``?mortar::`use-file-usgs` `` to view the function family's documentation and more examples.

`use_project_usgs()` generates all the common files for your repository in one function call: CHANGELOG.md, code.json, CODE_OF_CONDUCT.md, CONTRIBUTING.md, DISCLAMER.md, .gitignore, LICENSE.md, and README.md.

```{r eval=FALSE}
use_project_usgs(
  gitignore_additions = c("scratch*", "*/tmp/*"),
  readme_rmd = TRUE,
  disclaimer_approved = FALSE
)
```

Run `?mortar::use_project_usgs` to view the function's documentation and more examples.

## Set up a `targets` project with `tar_init()`

The `tar_init()` function will initialize a `targets` pipeline project directory according to the DS CoP conventions. Internal USGS users can see more on this in section 12 of the Targets 2 training course.
Run this function at the beginning of a `targets` pipeline project to avoid manually setting up the project directory (and forgetting to add those pesky `.empty` files in each sub-directory).

The call below illustrates how to use `tar_init()`.

```{r eval=FALSE}
tar_init(
  phase_names = c("fetch","process","summarize"),
  phase_subdirs = c("src","out"),
  use_leading_zeros = TRUE
)
```

Under the hood, `tar_init()` basically just repeatedly calls `dir.create()` and `file.create()` to accomplish the following tasks:

1. Creates a `_targets.R` file in the project's root directory with some general starter code.
2. Creates `.R` scripts and directories in the projects root directory for each phase of the pipeline like `01_fetch.R` and `01_fetch/`, etc.
3. For each phase directory, creates sub-directories therein, such as `src` and `out`.
4. Adds a `.empty` file within each sub-directory so that it will be tracked by git.

Run `?mortar::tar_init` to view the function's documentation and more examples.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tar_init.R
\name{tar_init}
\alias{tar_init}
\title{Initialize a targets project}
\usage{
tar_init(
  home,
  phase_names,
  phase_nums = seq_along(phase_names),
  separate_phase_scripts = TRUE,
  phase_subdirs = c("src", "out"),
  use_leading_zeros = FALSE,
  overwrite = FALSE
)
}
\arguments{
\item{home}{chr, root directory of targets project. To use the current
working directory, use  \code{"."}.}

\item{phase_names}{chr vector, names of target phases like "fetch",
"process", etc.}

\item{phase_nums}{int vector, numbers to prepend to phase_names like
\code{"1_fetch"}, \code{"2_process"}, etc. Defaults to
seq_along(phase_names)}

\item{separate_phase_scripts}{lgl, should a different R script be created for
each phase like "1_fetch.R", "2_process.R", etc? If FALSE, then targets
lists will be initialized in _targets.R file. Defaults to TRUE}

\item{phase_subdirs}{chr vector, subdirectories within each phase. Defaults
to "src", "out"}

\item{use_leading_zeros}{lgl, should "0" be used in front of phase numbers in
file and directory names?}

\item{overwrite}{lgl, should the initialization overwrite files and folders
that already exist? Defaults to FALSE}
}
\value{
\code{NULL} invisibly
}
\description{
Run this function in a new targets project directory. It will
create R files and directories using the targets project structure often
used by the USGS Data Science Community of Practice. For more information,
internal USGS employees can look at section 12 of Targets 2 training
course.
}
\examples{
# temporary directories in which targets project is initialized (you can skip
# this part if creating your own project)
tmp <- tempdir()
unlink(tmp, recursive = TRUE, force = TRUE)
dir.create(tmp)

# creates 1_fetch, 2_process, 3_summarize R scripts and directories
tar_init(home = tmp, phase_names = c("fetch", "process", "summarize"))

list.files(tmp, full.names = FALSE, recursive = TRUE, all.files = TRUE,
           pattern = "\\\\.(R|empty)$")

# clean out tmp folder
unlink(tmp, recursive = TRUE, force = TRUE)
dir.create(tmp)

# different structure starting with 0_config and including "in/" dir in each phase
tar_init(home = tmp,
         phase_names = c("config", "pull", "munge", "visualize"),
         phase_nums = 0:3,
         phase_subdirs = c("in", "src", "out"),
         separate_phase_scripts = FALSE)

list.files(tmp, full.names = FALSE, recursive = TRUE, all.files = TRUE,
           pattern = "\\\\.(R|empty)$")

# clean out tmp folder
unlink(tmp, recursive = TRUE, force = TRUE)
dir.create(tmp)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/use_project_usgs.R
\name{use-file-usgs}
\alias{use-file-usgs}
\alias{use_gitignore_usgs}
\alias{use_readme_usgs}
\alias{use_readme_rmd_usgs}
\alias{use_license_usgs}
\alias{use_code_json_usgs}
\alias{use_disclaimer_provisional_usgs}
\alias{use_disclaimer_approved_usgs}
\alias{use_code_of_conduct_usgs}
\alias{use_contributing_usgs}
\alias{use_changelog_usgs}
\alias{use_gitlab_mr_template_usgs}
\title{Add individual USGS project files to a directory}
\usage{
use_gitignore_usgs(home, additions = NULL, open = rlang::is_interactive())

use_readme_usgs(home, open = rlang::is_interactive())

use_readme_rmd_usgs(home, open = rlang::is_interactive())

use_license_usgs(home, open = rlang::is_interactive())

use_code_json_usgs(home, open = rlang::is_interactive())

use_disclaimer_provisional_usgs(home, open = rlang::is_interactive())

use_disclaimer_approved_usgs(home, open = rlang::is_interactive())

use_code_of_conduct_usgs(home, open = rlang::is_interactive())

use_contributing_usgs(home, repo_url = NULL, open = rlang::is_interactive())

use_changelog_usgs(home, open = rlang::is_interactive())

use_gitlab_mr_template_usgs(home, open = FALSE)
}
\arguments{
\item{home}{chr, root directory of targets project. To use the current
working directory, use  \code{"."}.}

\item{additions}{chr vector, other files/directories to be added to
.gitignore}

\item{open}{lgl; whether to open the file for interactive editing}

\item{repo_url}{chr, URL to Git repository. If \code{NULL} (default) a generic
URL will be provided as a link to the repository in CONTRIBUTING.md.
Otherwise, the issue page for \code{repo_url} will be used as the link.}
}
\value{
\code{NULL} invisibly
}
\description{
Creates common project files like .gitignore, README.md. LICENSE.md,
etc.#'
}
\examples{
tmp <- tempdir()

use_gitignore_usgs(home = tmp,
                   additions = c("excluded_file.R",
                                 "excluded_dir",
                                 "*excluded_pattern*"))
# here are the contents of the .gitignore:
cat(readLines(file.path(tmp,".gitignore")), sep = "\n")

unlink(tmp, recursive = TRUE, force = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/use_project_usgs.R
\name{use_project_usgs}
\alias{use_project_usgs}
\title{Set up a USGS project directory}
\usage{
use_project_usgs(
  home,
  gitignore_additions = NULL,
  readme_type = c("md", "rmd"),
  disclaimer_type = c("provisional", "approved"),
  repo_url = get_usgs_gitlab_url("origin"),
  use_mr_template = TRUE,
  open = rlang::is_interactive()
)
}
\arguments{
\item{home}{chr, root directory of targets project. To use the current
working directory, use  \code{"."}.}

\item{gitignore_additions}{chr vector, other files/directories to be added to
.gitignore or \code{NULL} (default) for no additions.}

\item{readme_type}{chr, either "md" to create README as a markdown file
(README.md; default) or "rmd" to create it as an R markdown file
(README.Rmd). Using an Rmd file will allow you to include R code and output
in your README.md.}

\item{disclaimer_type}{chr, either "provisional" (default) to include the
provisional software disclaimer statement or "approved" for the approved
disclaimer statement. It is important to only include the approved
disclaimer statement if your software is an approved software release.}

\item{repo_url}{chr, URL to Git repository. If \code{NULL} (default) a generic
URL will be provided as a link to the repository in CONTRIBUTING.md.
Otherwise, the issue page for \code{repo_url} will be used as the link.}

\item{use_mr_template}{lgl, Should GitLab Merge Request template be included
in project repository. Default is \code{TRUE}.}

\item{open}{lgl; whether to open the files for interactive editing.}
}
\value{
\code{NULL} invisibly
}
\description{
Wraps the other \code{use_*_usgs} functions to initialize
multiple files in one directory.
}
\examples{
tmp <- tempdir()
unlink(tmp, recursive = TRUE, force = TRUE)
dir.create(tmp)

use_project_usgs(
  home = tmp,
  repo_url = "https://code.usgs.gov/water/computational-tools/mortar"
)
list.files(tmp)

# start over
unlink(tmp, recursive = TRUE, force = TRUE)
dir.create(tmp)

# creates README.Rmd and DISCLAIMER_APPROVED instead
use_project_usgs(
  home = tmp,
  readme_type = "rmd",
  disclaimer_type = "approved",
  repo_url = "https://code.usgs.gov/water/computational-tools/mortar"
)
list.files(tmp)

# Clean up temp files
unlink(tmp, recursive = TRUE, force = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edit_file.R
\name{file_edit}
\alias{file_edit}
\title{Edit a text file programmatically}
\usage{
file_edit(file, txt, match, append = TRUE)
}
\arguments{
\item{file}{chr, path to a file}

\item{txt}{chr or chr fun, either a single character string to insert into
the file or a function that transforms a character string. If supplying a
function, the first argument of the function must accept a single string
and it must return a single string. The function will be applied
independently to each line in the file that matches the match argument.
Supports anonymous functions. See examples.}

\item{match}{num vector or lgl fun, either a numeric vector indicating line
indices in the file or a function that returns a logical indicating lines
to change. If supplying a function, the first argument of the function must
accept a single string and it must return a logical vector. The function
will be applied independently to each line in the file. Supports anonymous
functions. See examples.}

\item{append}{lgl, if TRUE the text will be added as a new line after the
line(s) specified in the match argument. If FALSE, then the text will
replace the lines specified in the match argument}
}
\value{
(invisibly) FALSE if no lines in the specified file match the match
argument and TRUE otherwise
}
\description{
Insert or transform (\code{txt} argument) one or more lines in a file specified
either by line number or a boolean function (\code{match} argument). Any file that
can be read with the \code{readLines()} function should be compatible.
}
\examples{
tmpfile <- tempfile()

writeLines(text =
             c("hello I am joe",
               "this is multiple lines of text",
               "12345",
               "",
               "^ line 4 is an empty line"),
           con = tmpfile)
cat(readLines(tmpfile),sep = "\n")

# The simplest use is to append a line in a file with new text. Note that txt
# must be a single character string.
file_edit(file = tmpfile,
          txt = "this line is appended after the second line",
          match = 2,
          append = TRUE)
cat(readLines(tmpfile),sep = "\n")

# We can instead replace a line using append = FALSE. Here, we replace the empty
# line in the file. After adding the line above, the empty line now occupies
# index 5.
file_edit(file = tmpfile,
          txt = "this line is no longer empty",
          match = 5,
          append = FALSE)
cat(readLines(tmpfile),sep = "\n")

# The txt argument can also be a function, anonymous or named. It just needs to
# accept a single string and return a single string. Here, we replace "is" with
# "was" in line 6.
file_edit(file = tmpfile,
          txt = ~ stringr::str_replace(.x, "is", "was"),
          match = 6,
          append = FALSE)
cat(readLines(tmpfile),sep = "\n")

# The match argument can also be a  function. It just needs to accept a single
# string and return a single boolean. You can use it with the txt argument
# for some interesting interactions. Here, we change every line containing
# the word "is" to uppercase
file_edit(file = tmpfile,
          txt = toupper,
          match = ~ stringr::str_detect(.x, " is "),
          append = FALSE)
cat(readLines(tmpfile),sep = "\n")

# strings containing carriage returns are preserved. Here, we use the readLines
# and length functions to ensure we append the string to the end of the file
multiline <-
"
Here is a string
that takes up multiple lines"

file_edit(file = tmpfile,
          txt = multiline,
          match = length(readLines(tmpfile)),
          append = TRUE)
cat(readLines(tmpfile),sep = "\n")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/use_project_usgs.R
\name{get_usgs_gitlab_url}
\alias{get_usgs_gitlab_url}
\title{Get the HTTPS URL for the git remote of a repo at code.usgs.gov}
\usage{
get_usgs_gitlab_url(remote_name = "origin")
}
\arguments{
\item{remote_name}{chr; name of the git remote. The default is "origin".}
}
\value{
a character sting of the HTTPS URL to the remote repo
}
\description{
Get the HTTPS URL for the git remote of a repo at code.usgs.gov
}
\examples{
\dontrun{
# Will fail if working directory is not in a git repo
get_usgs_gitlab_url("origin")
}
}
